package com.quickblox.quickblox_sdk.webrtc;

import android.graphics.SurfaceTexture;

import org.webrtc.EglBase;
import org.webrtc.EglRenderer;
import org.webrtc.GlRectDrawer;
import org.webrtc.RendererCommon;
import org.webrtc.ThreadUtils;
import org.webrtc.VideoFrame;

import java.util.concurrent.CountDownLatch;

/**
 * Created by Injoit on 2019-12-27.
 * Copyright © 2019 Quickblox. All rights reserved.
 */

/**
 * Display the video stream on a Surface.
 * renderFrame() is asynchronous to avoid blocking the calling thread.
 * This class is thread safe and handles access from potentially three different threads:
 * Interaction from the main app in init, release and setMirror.
 * Interaction from C++ rtc::VideoSinkInterface in renderFrame.
 * Interaction from SurfaceHolder lifecycle in surfaceCreated, surfaceChanged, and surfaceDestroyed.
 */
public class TextureRenderer extends EglRenderer {
    // Callback for reporting renderer events. Read-only after initilization so no lock required.
    private RendererCommon.RendererEvents rendererEvents;
    private final Object layoutLock = new Object();
    private boolean isFirstFrameRendered;

    /**
     * In order to render something, you must first call init().
     */
    public TextureRenderer(String name) {
        super(name);

    }

    public void init(final EglBase.Context sharedContext,
                     RendererCommon.RendererEvents rendererEvents) {
        init(sharedContext, rendererEvents, EglBase.CONFIG_PLAIN, new GlRectDrawer());
    }

    /**
     * Initialize this class, sharing resources with |sharedContext|. The custom |drawer| will be used
     * for drawing frames on the EGLSurface. This class is responsible for calling release() on
     * |drawer|. It is allowed to call init() to reinitialize the renderer after a previous
     * init()/release() cycle.
     */
    public void init(final EglBase.Context sharedContext,
                     RendererCommon.RendererEvents rendererEvents, final int[] configAttributes,
                     RendererCommon.GlDrawer drawer) {
        ThreadUtils.checkIsOnMainThread();
        this.rendererEvents = rendererEvents;
        synchronized (layoutLock) {
            isFirstFrameRendered = false;
        }
        super.init(sharedContext, configAttributes, drawer);
    }

    @Override
    public void init(final EglBase.Context sharedContext, final int[] configAttributes,
                     RendererCommon.GlDrawer drawer) {
        init(sharedContext, null /* rendererEvents */, configAttributes, drawer);
    }

    @Override
    public void onFrame(VideoFrame frame) {
        updateFrameDimensions(frame);
        super.onFrame(frame);
    }

    private SurfaceTexture texture;

    void surfaceCreated(final SurfaceTexture texture) {
        ThreadUtils.checkIsOnMainThread();
        this.texture = texture;
        createEglSurface(texture);
    }

    void surfaceDestroyed() {
        ThreadUtils.checkIsOnMainThread();
        final CountDownLatch completionLatch = new CountDownLatch(1);
        releaseEglSurface(completionLatch::countDown);
        ThreadUtils.awaitUninterruptibly(completionLatch);
    }

    private void updateFrameDimensions(VideoFrame frame) {
        synchronized (layoutLock) {
            if (!isFirstFrameRendered) {
                isFirstFrameRendered = true;
                texture.setDefaultBufferSize(frame.getRotatedWidth(), frame.getRotatedHeight());
            }
        }
    }
}
