package com.quickblox.quickblox_sdk.webrtc;

import android.graphics.SurfaceTexture;
import android.os.Build;

import androidx.annotation.Nullable;

import com.quickblox.videochat.webrtc.view.QBRTCVideoTrack;

import org.webrtc.EglBase;
import org.webrtc.RendererCommon;

import java.util.HashSet;
import java.util.Set;

import io.flutter.plugin.common.PluginRegistry;
import io.flutter.view.TextureRegistry;

/**
 * Created by Injoit on 2019-12-27.
 * Copyright © 2019 Quickblox. All rights reserved.
 */
class WebRTCRenderManager {

    private Set<RTCVideoRender> rtcRendersList = new HashSet<>();
    private WebRTCCallService callService;
    private final TextureRegistry textures;

    WebRTCRenderManager(PluginRegistry.Registrar registrar) {
        textures = registrar.textures();
    }

    void setCallService(WebRTCCallService callService) {
        this.callService = callService;
    }

    void releaseCallService() {
        callService = null;
    }

    boolean init(Integer userId) {
        RTCVideoRender rtcVideoRender = new RTCVideoRender(userId);
        return rtcRendersList.add(rtcVideoRender);
    }

    void startRendering(String sessionId, Integer userId, ResultCallback<Integer, String> resultCallback) {
        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender != null) {
            callService.getVideoTrack(sessionId, userId, new WebRTCCallService.ServiceCallback<QBRTCVideoTrack>() {
                @Override
                public void onSuccess(QBRTCVideoTrack videoTrack) {
                    TextureRegistry.SurfaceTextureEntry entry = textures.createSurfaceTexture();
                    SurfaceTexture surfaceTexture = entry.surfaceTexture();

                    int textureId = (int) entry.id();

                    rtcVideoRender.release();

                    EglBase.Context sharedContext = getRootEglBaseContext();
                    rtcVideoRender.init(sharedContext, null);

                    if (videoTrack.getRenderer() != null) {
                        videoTrack.removeRenderer(rtcVideoRender);
                    }

                    rtcVideoRender.surfaceCreated(surfaceTexture);

                    videoTrack.addRenderer(rtcVideoRender);

                    resultCallback.onSuccess(textureId);
                }

                @Override
                public void onError(String errorMessage) {
                    resultCallback.onError(errorMessage);
                }
            });
        }
    }

    void play(String sessionId, Integer userId, ResultCallback<Void, String> resultCallback) {
        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender != null) {
            callService.getVideoTrack(sessionId, userId, new WebRTCCallService.ServiceCallback<QBRTCVideoTrack>() {
                @Override
                public void onSuccess(QBRTCVideoTrack videoTrack) {
                    videoTrack.setEnabled(true);
                    resultCallback.onSuccess(null);
                }

                @Override
                public void onError(String errorMessage) {
                    resultCallback.onError(errorMessage);
                }
            });
        }
    }

    void pause(String sessionId, Integer userId, ResultCallback<Void, String> resultCallback) {
        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender != null) {
            callService.getVideoTrack(sessionId, userId, new WebRTCCallService.ServiceCallback<QBRTCVideoTrack>() {
                @Override
                public void onSuccess(QBRTCVideoTrack videoTrack) {
                    videoTrack.setEnabled(false);
                    resultCallback.onSuccess(null);
                }

                @Override
                public void onError(String errorMessage) {
                    resultCallback.onError(errorMessage);
                }
            });
        }
    }

    boolean setScaleType(Integer userId, RendererCommon.ScalingType scalingType) {
        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender == null) {
            return false;
        } else {
            float type;

            switch (scalingType) {
                case SCALE_ASPECT_FIT:
                    type = 1.0F;
                    break;
                case SCALE_ASPECT_FILL:
                    type = 0.0F;
                    break;
                case SCALE_ASPECT_BALANCED:
                    type = 0.5625F;
                    break;
                default: {
                    type = 0.0F;
                }
            }
            rtcVideoRender.setLayoutAspectRatio(type);
        }
        return true;
    }

    boolean release(Integer userId) {
        boolean result = false;

        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender != null) {
            rtcVideoRender.release();
            result = true;
            rtcRendersList.remove(new RTCVideoRender(userId));
        }
        return result;
    }

    boolean setMirror(Integer userId, boolean mirror) {
        boolean result = false;

        RTCVideoRender rtcVideoRender = getRTCView(userId);

        if (rtcVideoRender != null) {
            rtcVideoRender.setMirror(mirror);
            result = true;
        }
        return result;
    }

    private RTCVideoRender getRTCView(Integer userId) {
        RTCVideoRender rtcVideoRender = null;
        if (rtcRendersList.contains(new RTCVideoRender(userId))) {
            for (RTCVideoRender videoView : rtcRendersList) {
                if (videoView.userId.equals(userId)) {
                    rtcVideoRender = videoView;
                    break;
                }
            }
        }
        return rtcVideoRender;
    }

    private class RTCVideoRender extends TextureRenderer {
        private Integer userId;

        RTCVideoRender(Integer userId) {
            super("");
            this.userId = userId;
        }

        @Override
        public void release() {
            super.release();
            surfaceDestroyed();
        }

        @Override
        public boolean equals(@Nullable Object obj) {
            boolean equals;
            if (obj instanceof RTCVideoRender) {
                equals = this.userId.equals(((RTCVideoRender) obj).userId);
            } else {
                equals = super.equals(obj);
            }
            return equals;
        }

        @Override
        public int hashCode() {
            int hash = 3;
            hash = 53 * hash + userId.hashCode();
            return hash;
        }
    }

    private EglBase.Context getRootEglBaseContext() {
        EglBase rootEglBase;
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            rootEglBase = EglBase.createEgl10(EglBase.CONFIG_PLAIN);
        } else {
            rootEglBase = EglBase.create();
        }
        return rootEglBase.getEglBaseContext();
    }

    interface ResultCallback<R, E> {
        void onSuccess(R result);

        void onError(E error);
    }
}